@@ -35,13 +35,13 @@ compaudio_t compressor(struct compressor *self, compaudio_t signal, int skip_rms
 #ifndef GNUPLOT
    if (skip_rms)
 #endif
-      self->signal_db = gots = level2db(fabs(signal));
+      gots = level2db(fabs(signal));
 #ifndef GNUPLOT
    else
-      self->signal_db = gots = level2db(fabs(rms_calc(self->rms_filter, signal)));
+      gots = level2db(fabs(rms_calc(self->rms_filter, signal)));
 #endif
    if (!isfinite(gots))
-      self->signal_db = gots = -100.0F;
+      gots = -100.0F;
    else
       if (self->filter)
          {
@@ -98,29 +98,23 @@ compaudio_t compressor(struct compressor *self, compaudio_t signal, int skip_rms
 #endif
    /* Return the gain of the compressor in dB */
    return self->gain_db + self->opgain;
-   }   
+   }
 
 /* limiter: a basic hard knee compressor - called limiter because that is the mode */
 /* in which IDJC uses it */
 compaudio_t limiter(struct compressor *self, compaudio_t left, compaudio_t right) 
    {
-   compaudio_t gots, target_db, gain_target_db, ndb, diff;   
+   compaudio_t gots, gain_target_db, diff;
 
    gots = level2db(fabs((fabs(left) > fabs(right)) ? left : right));
    if (!isfinite(gots))
       gots = -100.0;
 
-   if (gots <= self->k2)
-      target_db = gots;
+   if (gots <= self->k1)
+      gain_target_db = 0.0F;
    else
-      target_db = self->k2;
-   if (gots > self->k1)
-      {
-      ndb = gots - self->k1;
-      target_db += ndb / self->ratio;
-      }            
-   gain_target_db = target_db - gots; 
-   
+      gain_target_db = (gots - self->k1) / self->ratio + self->k1 - gots;
+
    if (fabs(diff = gain_target_db - self->gain_db) > 0.0000004)
       {  
       if (self->gain_db > gain_target_db)
