@@ -33,6 +33,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <signal.h>
+#include <locale.h>
 
 #include "kvpparse.h"
 #include "dbconvert.h"
@@ -40,9 +41,10 @@
 #include "noisegate.h"
 #include "xlplayer.h"
 #include "ialloc.h"
-#include "askxine.h"
 #include "vorbinfo.h"
 #include "mp4tag.h"
+#include "sndfileinfo.h"
+#include "avcodecdecode.h"
 
 #define TRUE 1
 #define FALSE 0
@@ -91,8 +93,11 @@ int left_samples_total, right_samples_total;
 int main_play;
 /* if the main app segfaults or similar this counter will see that the mixer terminates */
 int timeout;
+int shutdown;
 /* flag set when jack closes the client thread for example when the jackd is closed */
 int jack_closed_f;
+/* flag to indicate whether to use the player reading function which supports speed variance */
+int speed_variance;
 /* buffers that process_audio uses when reading media player data */
 sample_t *lp_lc, *lp_rc, *rp_lc, *rp_rc, *jp_lc, *jp_rc, *ip_lc, *ip_rc;
 sample_t *lp_lcf, *lp_rcf, *rp_lcf, *rp_rcf, *jp_lcf, *jp_rcf, *ip_lcf, *ip_rcf;
@@ -208,13 +213,14 @@ struct xlplayer *plr_l, *plr_r, *plr_j, *plr_i; /* pipe reader instance stucture
 char *mixer_string, *compressor_string, *gate_string, *microphone_string;
 char *normalizer_string;
 char *micl, *micr, *auxl, *auxr, *audl, *audr, *strl, *strr, *action;
-char *oggpathname, *mp4pathname, *mp4taglist;
-char *playerpathname, *seek_s, *playerplaylist, *loop;
+char *oggpathname, *sndfilepathname, *mp4pathname, *mp4taglist, *avformatpathname;
+char *playerpathname, *seek_s, *size, *playerplaylist, *loop, *resamplequality;
 
 /* dictionary look-up type thing used by the parse routine */
 struct kvpdict kvpdict[] = {
 	 { "PLRP", &playerpathname },		/* The media-file pathname for playback */
          { "SEEK", &seek_s },			/* Playback initial seek time in seconds */
+         { "SIZE", &size },			/* Size of the file in seconds */
          { "PLPL", &playerplaylist },		/* A playlist for the media players */
          { "LOOP", &loop },			/* play in a loop */
          { "MIXR", &mixer_string },		/* Control strings */
@@ -232,7 +238,10 @@ struct kvpdict kvpdict[] = {
          { "STRR", &strr },
          { "OGGP", &oggpathname },
          { "MP4P", &mp4pathname },
+         { "SNDP", &sndfilepathname },
+         { "AVFP", &avformatpathname },
          { "MP4T", &mp4taglist },
+         { "RSQT", &resamplequality },
          { "ACTN", &action },			/* Action to take */
          { "", NULL }};
 
@@ -525,8 +534,7 @@ void update_mic_and_aux()	/* mic and aux mute/unmute smoothing */
       }
    }
 
-/* process_audio: the monsterific JACK callback routine */
-/* 17.5 pages of text with my monitor tilted to portrait mode */
+/* process_audio: the JACK callback routine */
 void process_audio(jack_nframes_t nframes)
    {
    int samples_todo;		/* The samples remaining counter in the main loop */
@@ -551,7 +559,17 @@ void process_audio(jack_nframes_t nframes)
    sample_t jp_lc_fade, jp_rc_fade, ip_lc_fade, ip_rc_fade;
 
    if (timeout > 8000)
-      exit(5);
+      {
+      if (!shutdown)
+         {
+         fprintf(stderr, "timeout exceeded - unblocking xlplayer threads, shutdown flag set\n");
+         shutdown = TRUE;
+         }
+      plr_l->command = CMD_COMPLETE;
+      plr_r->command = CMD_COMPLETE;
+      plr_j->command = CMD_COMPLETE;
+      plr_i->command = CMD_COMPLETE;
+      }
 
    /* get the data for the jack ports */
    la_buffer = lap = (sample_t *) jack_port_get_buffer (audio_left_port, nframes);
@@ -586,15 +604,25 @@ void process_audio(jack_nframes_t nframes)
    jp_rcf = jprcpf = irealloc(jp_rcf, nframes);
    ip_lcf = iplcpf = irealloc(ip_lcf, nframes);
    ip_rcf = iprcpf = irealloc(ip_rcf, nframes);
-   
+
    if (!(lp_lc && lp_rc && rp_lc && rp_rc && jp_lc && jp_rc && ip_lc && ip_rc &&
          lp_lcf && lp_rcf && rp_lcf && rp_rcf && jp_lcf && jp_rcf && ip_lcf && ip_rcf))
       {
-      printf("Malloc failure in process audio\n");
-      exit(5);
+      if (!shutdown)
+         {
+         printf("Malloc failure in process audio\n");
+         shutdown = TRUE;
+         }
+      plr_l->command = CMD_COMPLETE;
+      plr_r->command = CMD_COMPLETE;
+      plr_j->command = CMD_COMPLETE;
+      plr_i->command = CMD_COMPLETE;
       }  
 
-   read_from_player(plr_l, lp_lc, lp_rc, lp_lcf, lp_rcf, nframes);
+   if (speed_variance)
+      read_from_player_sv(plr_l, lp_lc, lp_rc, lp_lcf, lp_rcf, nframes);
+   else
+      read_from_player(plr_l, lp_lc, lp_rc, lp_lcf, lp_rcf, nframes);
    if (plr_l->have_swapped_buffers_f)
       {
       lp_lc_audf = lp_lc_aud * df;	/* volume levels of player at stoppage time */
@@ -603,8 +631,11 @@ void process_audio(jack_nframes_t nframes)
       lp_rc_strf = lp_rc_str * df; 
       }
    left_audio_runout = (plr_l->avail < player_samples_cutoff);
-   
-   read_from_player(plr_r, rp_lc, rp_rc, rp_lcf, rp_rcf, nframes);
+
+   if (speed_variance)
+      read_from_player_sv(plr_r, rp_lc, rp_rc, rp_lcf, rp_rcf, nframes);
+   else
+      read_from_player(plr_r, rp_lc, rp_rc, rp_lcf, rp_rcf, nframes);
    if (plr_r->have_swapped_buffers_f)
       {
       rp_lc_audf = rp_lc_aud * df;
@@ -721,7 +752,7 @@ void process_audio(jack_nframes_t nframes)
 	 noisegate_gain = noisegate(&left_mic_noisegate, digital_filter(ng_left_filter, lmic));
 	 /* apply the sidechain results to the input signal */
 	 
-        lmic *= db2level(compressor_gain + noisegate_gain);
+         lmic *= db2level(compressor_gain + noisegate_gain);
 	 
 	 /* do the same for the right microphone */	 
 	 compressor_gain = compressor(&right_mic_compressor, rmic, 0);
@@ -1478,12 +1509,6 @@ int process (jack_nframes_t nframes, void *arg)
    return 0;
    }
    
-int sample_rate_change()
-   {
-   fprintf(stderr, "The sample rate changed!  Exiting...\n");
-   exit(-1);
-   }
-   
 int peak_to_log(float peak)
    {
    if (peak <= 0.0)
@@ -1519,17 +1544,7 @@ void interrupt_handler(int data)	/* eats the first ^C */
       }
    }
 
-void askxine_segfault_handler(int data)
-   {
-   fprintf(stderr, "Segfaulted during attempted creation of the askxine pseudoplayer\n");
-   fprintf(stderr, "This is usually the result of a corrupt config file in ~/.xine\n");
-   fprintf(stderr, "Recommend that you delete your ~/.xine directory\n");
-   printf("Segmentation Fault\n");
-   fflush(stdout);
-   exit(5);
-   }
-   
-void regular_segfault_handler(int data)
+void segfault_handler(int data)
    {
    printf("Segmentation Fault\n");
    fflush(stdout);
@@ -1550,14 +1565,14 @@ void alarm_handler(int data)
        (cause = plr_i)->watchdog_timer++ == 4 ||
        (cause = plr_j)->watchdog_timer++ == 4) 
       {
-      if (cause->playmode == XL_INITIATE)
+      if (cause->playmode == PM_INITIATE)
          {
          cause->initial_audio_context = cause->current_audio_context;
-         cause->playmode = XL_STOPPED;
-         cause->command = XL_COMPLETE;
+         cause->playmode = PM_STOPPED;
+         cause->command = CMD_COMPLETE;
          }
       fprintf(stderr, "watchdog timer frozen for one of the media players -- possible bad media file\nshutting down the mixer in one second\n");
-      signal(SIGALRM, regular_segfault_handler);
+      signal(SIGALRM, segfault_handler);
       }
    alarm(1);
    }
@@ -1593,6 +1608,8 @@ int main(int argc, char **argv)
    int sync = FALSE;
    struct rms_calc *lm_fast_rms, *lm_slow_rms, *rm_fast_rms, *rm_slow_rms;
 
+   setenv("LC_ALL", "C", 1);			/* ensure proper sscanf operation */
+
    if (sigfillset(&mixersigset))
       {
       fprintf(stderr, "sigfillset failed\n");
@@ -1600,16 +1617,8 @@ int main(int argc, char **argv)
       }
    signal(SIGINT, interrupt_handler);
    signal(SIGALRM, alarm_handler); 
-   
-   /* the following can cause a segfault when the xine config file is corrupted in certain ways */
-   signal(SIGSEGV, askxine_segfault_handler);
-   if (!askxine_create())
-      {
-      printf("unable to create askxine player\n");
-      return 1;
-      }
-   signal(SIGSEGV, regular_segfault_handler);
-   
+   signal(SIGSEGV, segfault_handler);
+
    jack_set_error_function(my_jack_error_callback);
    client_name = (char *) malloc(9 * sizeof (char));
    strcpy(client_name, "idjc-mx");
@@ -1671,7 +1680,7 @@ int main(int argc, char **argv)
       printf("failed to create main player modules\n");
       return 1;
       }
-         
+   
    if (!(plr_j = xlplayer_create(sr, RB_SIZE, "jinglesplayer", &jack_closed_f)))
       {
       printf("failed to create jingles player module\n");
@@ -1811,7 +1820,7 @@ int main(int argc, char **argv)
       
    while (kvp_parse(kvpdict, fp))
       {
-      if (jack_closed_f == TRUE)
+      if (jack_closed_f == TRUE || shutdown == TRUE)
          break;
       if (!strcmp(action, "sync"))
          {
@@ -1821,28 +1830,25 @@ int main(int argc, char **argv)
          }
       if (sync == FALSE)
          continue;
-      
-      if (!strcmp(action, "askxine"))
-         askxine_getmetadata(playerpathname);
-      
+
       if (!strcmp(action, "playleft"))
          {
-         fprintf(stdout, "context_id=%d\n", xlplayer_play(plr_l, playerpathname, atoi(seek_s)));
+         fprintf(stdout, "context_id=%d\n", xlplayer_play(plr_l, playerpathname, atoi(seek_s), atoi(size)));
          fflush(stdout);
          }
       if (!strcmp(action, "playright"))
          {
-         fprintf(stdout, "context_id=%d\n", xlplayer_play(plr_r, playerpathname, atoi(seek_s)));
+         fprintf(stdout, "context_id=%d\n", xlplayer_play(plr_r, playerpathname, atoi(seek_s), atoi(size)));
          fflush(stdout);
          }
       if (!strcmp(action, "playnoflushleft"))
          {
-         fprintf(stdout, "context_id=%d\n", xlplayer_play_noflush(plr_l, playerpathname, atoi(seek_s)));
+         fprintf(stdout, "context_id=%d\n", xlplayer_play_noflush(plr_l, playerpathname, atoi(seek_s), atoi(size)));
          fflush(stdout);
          }
       if (!strcmp(action, "playnoflushright"))
          {
-         fprintf(stdout, "context_id=%d\n", xlplayer_play_noflush(plr_r, playerpathname, atoi(seek_s)));
+         fprintf(stdout, "context_id=%d\n", xlplayer_play_noflush(plr_r, playerpathname, atoi(seek_s), atoi(size)));
          fflush(stdout);
          }
       
@@ -1882,11 +1888,16 @@ int main(int argc, char **argv)
          xlplayer_dither(plr_i, FALSE);
          }
       
+      if (!strcmp(action, "resamplequality"))
+         {
+         plr_l->rsqual = plr_r->rsqual = plr_j->rsqual = plr_i->rsqual = resamplequality[0] - '0';
+         }
+      
       if (!strcmp(action, "ogginforequest"))
          {
          if (vorbinfo(oggpathname, &artist, &title, &length))
             {
-            fprintf(stdout, "OIR:ARTIST=%s\nOIR:TITLE=%s\nOIR:LENGTH=%lf\nOIR:end\n", artist, title, length);
+            fprintf(stdout, "OIR:ARTIST=%s\nOIR:TITLE=%s\nOIR:LENGTH=%f\nOIR:end\n", artist, title, length);
             fflush(stdout);
             }
          else
@@ -1895,6 +1906,9 @@ int main(int argc, char **argv)
             fflush(stdout);
             }
          }
+      
+      if (!strcmp(action, "sndfileinforequest"))
+         sndfileinfo(sndfilepathname);
 
 #ifdef HAVE_MP4FF
       if (!strcmp(action, "mp4inforequest"))
@@ -1905,6 +1919,13 @@ int main(int argc, char **argv)
          mp4_tag_write(mp4pathname, mp4taglist);
 #endif
 
+#ifdef HAVE_AVCODEC
+#ifdef HAVE_AVFORMAT
+      if (!strcmp(action, "avformatinforequest"))
+         avformatinfo(avformatpathname);
+#endif
+#endif
+
       if (!strcmp(action, "micmixstats"))
          {
 	 if (sscanf(microphone_string, ":%d:%d:%d:%d:%d:%d:%d:%f:", &mic_filter,
@@ -1995,10 +2016,10 @@ int main(int argc, char **argv)
       if (!strcmp(action, "mixstats"))
          {
          if(sscanf(mixer_string,
-                ":%03d:%03d:%03d:%03d:%03d:%d:%1d%1d%1d%1d%1d:%1d%1d:%1d%1d%1d:%1d%1d%1d%1d:%1d:%1d:%1d:%1d:%1d:",
+                ":%03d:%03d:%03d:%03d:%03d:%d:%1d%1d%1d%1d%1d:%1d%1d:%1d%1d%1d:%1d%1d%1d%1d:%1d:%1d:%1d:%1d:%1d:%f:%f:%1d:",
       		&volume, &crossfade, &jinglesvolume, &interludevol, &mixbackvol, &jingles_playing,
                 &left_stream, &left_audio, &right_stream, &right_audio, &stream_monitor,
-		&new_left_pause, &new_right_pause, &lmic_on, &rmic_on, &aux_on, &flush_left, &flush_right, &flush_jingles, &flush_interlude, &simple_mixer, &eot_alarm_f, &mixermode, &fadeout_f, &main_play)!=25)
+		&new_left_pause, &new_right_pause, &lmic_on, &rmic_on, &aux_on, &flush_left, &flush_right, &flush_jingles, &flush_interlude, &simple_mixer, &eot_alarm_f, &mixermode, &fadeout_f, &main_play, &(plr_l->newpbspeed), &(plr_r->newpbspeed), &speed_variance) !=28)
 	    {
 	    fprintf(stderr, "mixer got bad mixer string\n");
 	    break;
@@ -2193,7 +2214,7 @@ int main(int argc, char **argv)
                          plr_j->current_audio_context,
                          plr_i->current_audio_context,
             		 left_audio_runout && (!(plr_l->current_audio_context & 0x1)),
-            		 right_audio_runout && (!(plr_l->current_audio_context & 0x1)));
+            		 right_audio_runout && (!(plr_r->current_audio_context & 0x1)));
 	 /* tell the jack mixer it can reset its vu stats now */
 	 reset_vu_stats_f = TRUE;
 	 fflush(stdout);
@@ -2223,7 +2244,6 @@ int main(int argc, char **argv)
    ifree(jp_rcf);
    ifree(ip_lcf);
    ifree(ip_rcf);
-   askxine_destroy();
    xlplayer_destroy(plr_l);
    xlplayer_destroy(plr_r);
    xlplayer_destroy(plr_j);
