@@ -107,6 +107,8 @@ char ai_samplerate[8];
 char ai_channels[2];
 /* stops flush from blocking when jack is not running */
 int jack_closed_f;
+/* set when cpu overloaded or network is sluggish to indicate frames have been dropped */
+int stream_buffer_full;
 
 struct kvpdict kvpdict[] = {
          { "SERV", &server_type },
@@ -165,6 +167,10 @@ void process_audio(jack_nframes_t nframes)
    sample_t *ptrl, *ptrr;
    sample_t *ls_buffer = (sample_t *) jack_port_get_buffer (stream_left_port, nframes);
    sample_t *rs_buffer = (sample_t *) jack_port_get_buffer (stream_right_port, nframes);
+#if HAVE_LIBSHOUT
+   jack_nframes_t space_avail, todo;
+#endif
+   jack_nframes_t sspace_avail, stodo;
    
    /* audio level analysis for the autopause feature */
    ptrl = ls_buffer + nframes;
@@ -178,13 +184,18 @@ void process_audio(jack_nframes_t nframes)
       }
 
 #if HAVE_LIBSHOUT
-   jack_nframes_t space_avail = jack_ringbuffer_write_space(rightring) / sizeof (sample_t);
-   jack_nframes_t todo = (space_avail > nframes ? nframes : space_avail);
+   space_avail = jack_ringbuffer_write_space(rightring) / sizeof (sample_t);
+   todo = (space_avail > nframes ? nframes : space_avail);    
     
    if (ringbuffer_can_write == TRUE)
       {
-      jack_ringbuffer_write(leftring, (char *)ls_buffer, todo * sizeof (sample_t));
-      jack_ringbuffer_write(rightring, (char *)rs_buffer, todo * sizeof (sample_t));
+      if (todo)
+         {
+         jack_ringbuffer_write(leftring, (char *)ls_buffer, todo * sizeof (sample_t));
+         jack_ringbuffer_write(rightring, (char *)rs_buffer, todo * sizeof (sample_t));
+         }
+      else
+         stream_buffer_full = TRUE;
       }
 
    if (ringbuffer_flush == TRUE)
@@ -195,8 +206,8 @@ void process_audio(jack_nframes_t nframes)
       }   
 #endif /* HAVE_LIBSHOUT */
 
-   jack_nframes_t sspace_avail = jack_ringbuffer_write_space(rightsring) / sizeof (sample_t);
-   jack_nframes_t stodo = (sspace_avail > nframes ? nframes : sspace_avail);
+   sspace_avail = jack_ringbuffer_write_space(rightsring) / sizeof (sample_t);
+   stodo = (sspace_avail > nframes ? nframes : sspace_avail);
 
    if (sringbuffer_flush == TRUE)
       {
@@ -245,7 +256,8 @@ void *save_codec_feeder(void *args)
    size_t frames;
    sample_t *lbuff, *rbuff, *l, *r;
    unsigned char *p, *pcmbuff;
-   int pcmvalue;
+   int32_t pcmvalue;
+   float scaled_value;
    int i, leftmute = FALSE, rightmute = FALSE, lefthcp = 0, righthcp = 0, begin = 0, end = 0, zc_timeout = 0;
    enum crosstype { ZC_NORMAL, ZC_CLOSING, ZC_PAUSED, ZC_OPENING };
    enum crosstype zerocross = ZC_NORMAL;
@@ -261,7 +273,7 @@ void *save_codec_feeder(void *args)
       fprintf(stderr, "Unable to allocate right buffer in save_codec_feeder\n");
       keep_saving = FALSE;
       }
-   if(!(pcmbuff = (unsigned char *)malloc(4 * SRC_SIZE)))
+   if(!(pcmbuff = (unsigned char *)malloc(8 * SRC_SIZE)))
       {
       fprintf(stderr, "Unable to allocate space for pcm buffer\n");
       keep_saving = FALSE;
@@ -365,16 +377,60 @@ void *save_codec_feeder(void *args)
       l=lbuff;
       r=rbuff;
       p=pcmbuff;
-      for (i = begin; i < end; i++)
+      
+      
+      if (!strcmp(save_format, "mp3"))
          {
-         pcmvalue = floorf(*l++ * 32767.5);
-         *p++ = pcmvalue & 0xFF;
-         *p++ = (pcmvalue >> 8) & 0xFF;
-         pcmvalue = floorf(*r++ * 32767.5);
-         *p++ = pcmvalue & 0xFF;
-         *p++ = (pcmvalue >> 8) & 0xFF;
+         for (i = begin; i < end; i++)
+            {
+            scaled_value = *l++ * 2147483648.0F;
+            pcmvalue = (int32_t)scaled_value;
+            if (scaled_value > 2147483647.0F)
+               pcmvalue = 0x7FFFFFFF;
+            if (scaled_value < -2147483648.0F)
+               pcmvalue = 0xFFFFFFFF;
+            *p++ = pcmvalue;
+            *p++ = pcmvalue >> 8;
+            *p++ = pcmvalue >> 16;
+            *p++ = pcmvalue >> 24;
+   
+            scaled_value = *r++ * 2147483648.0F;
+            pcmvalue = (int32_t)scaled_value;
+            if (scaled_value > 2147483647.0F)
+               pcmvalue = 0x7FFFFFFF;
+            if (scaled_value < -2147483648.0F)
+               pcmvalue = 0xFFFFFFFF;
+            *p++ = pcmvalue;
+            *p++ = pcmvalue >> 8;
+            *p++ = pcmvalue >> 16;
+            *p++ = pcmvalue >> 24;
+            }
+         write(swriter[1], pcmbuff, frames * 8);
+         }
+      else
+         {
+         for (i = begin; i < end; i++)
+            {
+            scaled_value = *l++ * 32768;
+            pcmvalue = (int32_t)scaled_value;
+            if (scaled_value > 32767)
+               scaled_value = 32767;
+            if (scaled_value < -32768)
+               scaled_value = -32768;
+            *p++ = pcmvalue;
+            *p++ = pcmvalue >> 8;
+            
+            scaled_value = *r++ * 32768;
+            pcmvalue = (int32_t)scaled_value;
+            if (scaled_value > 32767)
+               scaled_value = 32767;
+            if (scaled_value < -32768)
+               scaled_value = -32768;
+            *p++ = pcmvalue;
+            *p++ = pcmvalue >> 8;
+            }
+         write(swriter[1], pcmbuff, frames * 4);
          }
-      write(swriter[1], pcmbuff, frames * 4);
       }
    
    paused = 0;
@@ -623,7 +679,8 @@ void *codec_feeder(void *args)
    size_t frames;
    sample_t *lbuff, *rbuff, *l, *r;
    unsigned char *p, *pcmbuff;
-   int pcmvalue;
+   int32_t pcmvalue;
+   float scaled_value;
    int i;
    
    usleep(20000);
@@ -637,12 +694,12 @@ void *codec_feeder(void *args)
       fprintf(stderr, "Unable to allocate right buffer in codec_feeder\n");
       keep_connection = FALSE;
       }
-   if(!(pcmbuff = (unsigned char *)malloc(4 * SRC_SIZE)))
+   if(!(pcmbuff = (unsigned char *)malloc(8 * SRC_SIZE)))
       {
       fprintf(stderr, "Unable to allocate space for pcm buffer\n");
       keep_connection = FALSE;
       }
-      
+
    ringbuffer_can_write = TRUE;		/* Turn on the flow of data from JACK */     
    
    while (keep_connection == TRUE)
@@ -658,14 +715,29 @@ void *codec_feeder(void *args)
       p=pcmbuff;
       for (i = 0; i < frames; i++)
          {
-	 pcmvalue = floorf(*l++ * 32767.5);
-	 *p++ = pcmvalue & 0xFF;
-	 *p++ = (pcmvalue >> 8) & 0xFF;
-	 pcmvalue = floorf(*r++ * 32767.5);
-	 *p++ = pcmvalue & 0xFF;
-	 *p++ = (pcmvalue >> 8) & 0xFF;
+         scaled_value = *l++ * 2147483648.0F;	/* as below but retaining at least 24 bits of precision */
+         pcmvalue = (int32_t)scaled_value;
+         if (scaled_value > 2147483647.0F)
+            pcmvalue = 0x7FFFFFFF;
+         if (scaled_value < -2147483648.0F)
+            pcmvalue = 0xFFFFFFFF;
+         *p++ = pcmvalue;
+         *p++ = pcmvalue >> 8;
+         *p++ = pcmvalue >> 16;
+         *p++ = pcmvalue >> 24;
+
+         scaled_value = *r++ * 2147483648.0F;
+         pcmvalue = (int32_t)scaled_value;
+         if (scaled_value > 2147483647.0F)
+            pcmvalue = 0x7FFFFFFF;
+         if (scaled_value < -2147483648.0F)
+            pcmvalue = 0xFFFFFFFF;
+         *p++ = pcmvalue;
+         *p++ = pcmvalue >> 8;
+         *p++ = pcmvalue >> 16;
+         *p++ = pcmvalue >> 24;
          }
-      write(writer[1], pcmbuff, frames * 4);
+      write(writer[1], pcmbuff, frames * 8);
       }
    
    ringbuffer_can_write = FALSE;
@@ -1052,12 +1124,12 @@ int stream_connect()
             if (bitrate >= 64)
                {
                qualval = highquality ? "1" : "2";
-               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-", NULL);
+               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "--bitwidth", "32", "-", NULL);
                }
             else
                {
                qualval = highquality ? "0" : "2";
-               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "--resample", resamplerate, "-", NULL);
+               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "--resample", resamplerate, "--bitwidth", "32", "-", NULL);
                }
 	    }
          else
@@ -1065,12 +1137,12 @@ int stream_connect()
             if (bitrate >= 64)
                {
                qualval = highquality ? "1" : "2";
-               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-m", "s", "-a", "-", NULL);
+               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-m", "s", "-a", "--bitwidth", "32", "-", NULL);
                }
             else
                {
                qualval = highquality ? "0" : "2";
-               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-m", "s", "-a", "--resample", resamplerate, "-", NULL);
+               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-m", "s", "-a", "--resample", resamplerate, "--bitwidth", "32", "-", NULL);
                }
 	    }
 	 fprintf(stderr, "Error: execlp failed to spawn lame process or bad option\n");
@@ -1264,11 +1336,11 @@ int start_saving()
          fprintf(stderr, "mp3 chosen\n");
 	 if (bitrate >= 64)
             {
-            execlp(getenv("lame"), "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "-b", save_bitrate, "--tt", save_title, "--ta", save_artist, "-", NULL);
+            execlp(getenv("lame"), "lame", "--bitwidth", "32", "-r", "-x", "-h", "-s", samplerate, "--cbr", "-b", save_bitrate, "--tt", save_title, "--ta", save_artist, "-", NULL);
             }
          else
             {
-            execlp(getenv("lame"), "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "-b", save_bitrate, "--tt", save_title, "--ta", save_artist, "--resample", resamplerate, "-", NULL);
+            execlp(getenv("lame"), "lame", "--bitwidth", "32", "-r", "-x", "-h", "-s", samplerate, "--cbr", "-b", save_bitrate, "--tt", save_title, "--ta", save_artist, "--resample", resamplerate, "-", NULL);
             }
 	 }
       else if (!strcmp(save_format, "ogg"))
@@ -1339,6 +1411,8 @@ int main(int argc, char **argv)
    FILE *fp = stdin;
    char *client_name;
 
+   setenv("LC_ALL", "C", 1);
+
    if(sigfillset(&serversigset))
       {
       fprintf(stderr, "sigfillset failed\n");
@@ -1471,6 +1545,11 @@ int main(int argc, char **argv)
 	 printf("IDJCserver: %s %s %lu %d %d\n", reply[connection], status_tag, total, is_saving, paused);
 	 fflush(stdout);
 	 }
+      if (stream_buffer_full)
+         {
+         stream_buffer_full = FALSE;
+         fprintf(stderr, "idjcserver: audio data lost - check CPU load and network connection\n");
+         }
       }
    jack_client_close(client);
 #if HAVE_LIBSHOUT
