@@ -46,7 +46,7 @@ int dict_apply_to_target(struct kvpdict *dp, char *key, char *target)
    int append;
    size_t origtext_siz, newtext_siz;
    
-   if (append = (key[0] == '+'))	/* If key starts with a plus we will not replace -- we will append */
+   if ((append = (key[0] == '+')))	/* If key starts with a plus we will not replace -- we will append */
       ++key;
 
    for (; dp->target; dp++)		/* Iterate over NULL terminated dictionary */
