@@ -27,7 +27,7 @@ struct rms_calc
    rmsaudio_t *end;
    rmsaudio_t *ptr;
    rmsaudio_t sum;
-   rmsaudio_t newsum;		/* value used to prevent drift */
+   rmsaudio_t newsum;		/* provides drift correction */
    rmsaudio_t multiplier;
    };
    
