@@ -1,6 +1,6 @@
 /*
-#   xlplayer.h: a xine-lib based player module for idjc
-#   Copyright (C) 2006 Stephen Fairchild
+#   xlplayer.h: player decoder module for idjc
+#   Copyright (C) 2006-2007 Stephen Fairchild
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -17,19 +17,34 @@
 #   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
+#ifndef XLPLAYER_H
+#define XLPLAYER_H
+
+#include "../config.h"
 #include <jack/jack.h>
 #include <jack/ringbuffer.h>
 #include <pthread.h>
 #include <stdlib.h>
 #include <samplerate.h>
-#include <FLAC/all.h>
 #include <sndfile.h>
+#include <vorbis/vorbisfile.h>
 
-enum command_t {XL_COMPLETE, XL_PLAY, XL_EJECT, XL_CLEANUP, XL_THREADEXIT, XL_PLAYMANY};
+#ifdef HAVE_FLAC
+#include <FLAC/all.h>
+#endif
 
-enum playmode_t {XL_STOPPED, XL_INITIATE, XL_PLAYING, XL_EJECTING };
+#ifdef HAVE_FAAD
+#include <mp4ff.h>
+#include <faad.h>
+#endif
 
-enum playertype_t {XL_XINE, XL_VORBIS, XL_SNDFILE, XL_FLAC, XL_BADFILE};
+#ifdef HAVE_MAD
+#include <mad.h>
+#endif
+
+enum command_t {CMD_COMPLETE, CMD_PLAY, CMD_EJECT, CMD_CLEANUP, CMD_THREADEXIT, CMD_PLAYMANY};
+
+enum playmode_t {PM_STOPPED, PM_INITIATE, PM_PLAYING, PM_EJECTING };
 
 struct xlplayer
    {
@@ -43,6 +58,7 @@ struct xlplayer
    char **playlist;			/* the playlist as an array of pointers */
    int loop;				/* flag indicating if we loop or come to a stop */
    int seek_s;				/* the initial seek time of the song in seconds */
+   int size;				/* size of the file in seconds */
    int playlistmode;			/* set when we are using a local playlist */
    int playlistindex;			/* current track number we are playing */
    int playlistsize;			/* the number of tracks in the playlist */
@@ -58,30 +74,42 @@ struct xlplayer
    u_int64_t samples_written;		/* number of samples written to the ringbuffer */
    int32_t play_progress_ms;		/* the playback progress in milliseconds */
    int32_t fadein_index;		/* used for fadein when seeking */
-   char *playername;			/* identifies this particular instance -- for error messages */
-   enum playmode_t playmode;		/* indicates the player mode */
+   char *playername;			/* the name of this player e.g. "left", "right" etc. */
+   enum playmode_t playmode;		/* indicates the player mode or state */
    enum command_t command;		/* the command mode */
-   int test2;
    int have_swapped_buffers_f;		/* controls buffer swapping for fadeout */
    size_t avail;			/* the number of samples available in the ringbuffer */
    int have_data_f;			/* indicates the presence of audio data */
    int current_audio_context;		/* bumps when started, bumps when stopped. Odd=playing */
    int initial_audio_context;		/* return code placeholder variable for above */
-   int dither;				/* flag, whether to add dither to player output */
+   int dither;				/* whether to add dither to player output FLAC, MP4, WAV only */
    unsigned int seed;			/* used for dither */
-   pthread_t thread;
+   pthread_t thread;			/* thread pointer for the player main loop */
    u_int32_t sleep_samples;		/* used to count off when it is appropriate to call sleep */
-   SRC_STATE *src_state;		/* give flac access to the resampler */
-   SRC_DATA *src_data;
-   FLAC__uint64 totalsamples;		/* used to determine the last frame */
-   float *flbuf;
-   int noflush;				/* suppresses ringbuffer flushes */
-   int suppress_flac_output;		/* used to suppress flac output so preventing odd seek noise */
+   SRC_STATE *src_state;		/* used by resampler */
+   SRC_DATA src_data;
+   int rsqual;				/* resample quality */   
+   int noflush;				/* suppresses ringbuffer flushes for gapless playback */
    int *jack_shutdown_f;		/* inidcator that jack has shut down */
-   int watchdog_timer;			/* this will run away if the main loop has frozen */
-   SNDFILE *sndfile;			/* file handle from libsndfile for handling wav type files */
-   SF_INFO sf_info;			/* points to data about current libsndfile frame */
+   int watchdog_timer;
    int up;				/* set to true when the player is fully initialised */
+   double pbspeed;			/* the playback speed as a factor */
+   float newpbspeed;			/* the value the above is updated with */
+   SRC_STATE *pbspeed_conv_l;		/* libsamplerate handle for playback speed control - left channel */
+   SRC_STATE *pbspeed_conv_r;
+   SRC_STATE *pbspeed_conv_lf;		/* as above but for fade buffer */
+   SRC_STATE *pbspeed_conv_rf;
+   float *pbsrb_l;			/* input buffers for the playback speed converter */
+   float *pbsrb_r;
+   float *pbsrb_lf;
+   float *pbsrb_rf;
+   long pbs_norm_read_qty;		/* the number of normal samples which will be read from left and right channels */
+   long pbs_fade_read_qty;		/* the number of fadeout samples which will be read */
+   int pbs_exchange;			/* keeps correct association for input buffers after a buffer swap occurs */
+   void *dec_data;			/* points to audio decoder data */
+   void (*dec_init)(struct xlplayer *);	/* audio decoder init function */
+   void (*dec_play)(struct xlplayer *);	/* function that decodes one frame of audio data */
+   void (*dec_eject)(struct xlplayer *);/* function that cleans up after the decoder */
    };
 
 /* xlplayer_create: create an instance of the player */
@@ -92,7 +120,7 @@ void xlplayer_destroy(struct xlplayer *);
 /* xlplayer_play: starts the player on a particular track immediately
 * if a track is currently playing eject is called
 * return value: a context-id for this track */
-int xlplayer_play(struct xlplayer *self, char *pathname, int seek_s);
+int xlplayer_play(struct xlplayer *self, char *pathname, int seek_s, int size);
 
 /* xlplayer_playmany: starts the player on a playlist
 * if a track is currently playing eject is called, also can set looping with this function
@@ -100,7 +128,7 @@ int xlplayer_play(struct xlplayer *self, char *pathname, int seek_s);
 int xlplayer_playmany(struct xlplayer *self, char *playlist, int loop_f);
 
 /* xlplayer_play_noflush: starts the player without flushing out old data from the ringbuffer */
-int xlplayer_play_noflush(struct xlplayer *self, char *pathname, int seek_s);
+int xlplayer_play_noflush(struct xlplayer *self, char *pathname, int seek_s, int size);
 
 /* xlplayer_cancelplaynext: cancels the automatic playing of the next track 
 * the current track is allowed to continue playing */
@@ -121,9 +149,27 @@ void xlplayer_eject(struct xlplayer *self);
 /* this is meant to be run inside the jack callback */
 size_t read_from_player(struct xlplayer *self, jack_default_audio_sample_t *left_buf, jack_default_audio_sample_t *right_buf, jack_default_audio_sample_t *left_fbuf, jack_default_audio_sample_t *right_fbuf, jack_nframes_t nframes);
 
+/* read_from_player_sv: reads out the audio data from the buffers but provides speed variance (pitch control) */
+/* this is meant to be run inside the jack callback */
+size_t read_from_player_sv(struct xlplayer *self, jack_default_audio_sample_t *left_buf, jack_default_audio_sample_t *right_buf, jack_default_audio_sample_t *left_fbuf, jack_default_audio_sample_t *right_fbuf, jack_nframes_t nframes);
+
 /* xlplayer_create_fadein_lookup: creates the lookup table used for song fadein */
 /* must be called before starting any of the players */
 int xlplayer_create_fadein_lookup(float size_seconds, int samplerate);
 
 /* xlplayer_destroy_fadein_lookup: frees the lookup table used for fading */
 void xlplayer_destroy_fadein_lookup(void);
+
+/* calculate the gain for fading in - used when seeking to prevent clicks */
+inline jack_default_audio_sample_t xlplayer_get_next_gain(struct xlplayer *self);
+
+/* put audio data in format recognised by jack and libsamplerate */
+float *xlplayer_make_audio_to_float(struct xlplayer *self, float *buffer, uint8_t *data, int num_samples, int bits_per_sample, int num_channels);
+
+/* splits audio data into separate audio streams, ready for writing */
+void xlplayer_demux_channel_data(struct xlplayer *self, jack_default_audio_sample_t *buffer, int num_samples, int num_channels);
+
+/* cause the cached pcm data to be written out to the jack ringbuffer */
+void xlplayer_write_channel_data(struct xlplayer *self);
+
+#endif /* XLPLAYER_H */
