@@ -109,6 +109,60 @@ def make_entry_line(parent, item, code):
    return box, checkbox, entry, button
 
 class mixprefs:
+   class event_command_container(gtk.Frame):
+      def add(self, widget):
+         self.vbox.add(widget)
+      def __init__(self):
+         gtk.Frame.__init__(self)
+         gtk.Frame.set_border_width(self, 4)
+         gtk.Frame.set_shadow_type(self, gtk.SHADOW_ETCHED_OUT)
+         self.vbox = gtk.VBox()
+         self.vbox.set_spacing(2)
+         gtk.Frame.add(self, self.vbox)
+         self.vbox.set_border_width(4)
+         self.vbox.show()
+   
+   class event_command(gtk.HBox):
+      def activate(self):
+         if self.checkbutton.get_active():
+            os.system(self.entry.get_text())
+      def get_text(self):
+         return self.entry.get_text()
+      def get_active(self):
+         return self.checkbutton.get_active()
+      def set_text(self, text):
+         return self.entry.set_text(text)
+      def set_active(self, bool):
+         return self.checkbutton.set_active(bool)
+      def cb_checkbutton(self, widget, data = None):
+         self.entry.set_sensitive(widget.get_active())
+      def __init__(self, imagefile, width, height, text, default_state, crossout, tips = None, checkbutton_tip = None, entry_tip = None):
+         gtk.HBox.__init__(self)
+         gtk.HBox.set_spacing(self, 6)
+         self.checkbutton = gtk.CheckButton()
+         self.checkbutton.set_active(default_state)
+         image = gtk.Image()
+         if crossout:
+            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(pkgdatadir + "crossout" + gfext, width , height)
+         else:
+            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(pkgdatadir + imagefile + gfext, width, height)
+         image.set_from_pixbuf(pixbuf)
+         self.checkbutton.add(image)
+         image.show()
+   	 gtk.HBox.pack_start(self, self.checkbutton, False, False, 0)
+         self.checkbutton.connect("toggled", self.cb_checkbutton)
+         self.checkbutton.show()
+         self.entry = gtk.Entry()
+         self.entry.set_text(text)
+         gtk.HBox.pack_start(self, self.entry, True, True, 0)
+         self.entry.set_sensitive(default_state)
+         self.entry.show()
+         if tips is not None:
+            if checkbutton_tip is not None:
+               tips.set_tip(self.checkbutton, checkbutton_tip)
+            if entry_tip is not None:
+               tips.set_tip(self.entry, entry_tip)
+   
    def send_new_compressor_stats(self):
       string_to_send = ":%.1f:%d:%d:%d:%d:%d:%d:%.1f:%d:%.1f:%d:" % (
       					self.ratio_adj.get_value(),
@@ -158,6 +212,9 @@ class mixprefs:
                                         self.normalize.get_active())
       self.parent.mixer_write("NORM=%s\nACTN=normalizerstats\nend\n" % string_to_send, True)
       
+   def send_new_resampler_stats(self):
+      self.parent.mixer_write("RSQT=%d\nACTN=resamplequality\nend\n" % self.resample_quality, True)
+      
    def normalizer_defaults(self, value = None):
       self.normboost_adj.set_value(12.0)
       self.normboost_adj.value_changed()
@@ -171,6 +228,11 @@ class mixprefs:
    def cb_normalizer(self, widget, data = None):
       self.send_new_normalizer_stats()
       
+   def cb_resample_quality(self, widget, data):
+      if widget.get_active():
+         self.resample_quality = data
+         self.send_new_resampler_stats()
+      
    def cb_dither(self, widget, data = None):
       if widget.get_active():
          string_to_send = "ACTN=dither\nend\n"
@@ -439,6 +501,17 @@ class mixprefs:
          entry.insert_text("," + colour, cursor)	# Background
       entry.set_position(cursor + 3)
 
+   def cb_pbspeed(self, widget):
+      if widget.get_active():
+         self.parent.player_left.pbspeedbar.set_value(0)
+         self.parent.player_right.pbspeedbar.set_value(0)
+         self.parent.player_left.pbspeedbox.show()
+         self.parent.player_right.pbspeedbox.show()
+      else:
+         self.parent.player_left.pbspeedbox.hide()
+         self.parent.player_right.pbspeedbox.hide()
+      self.parent.send_new_mixer_stats()
+
    def colourmenupopulate(self, entry, menu):
       menusep = gtk.SeparatorMenuItem()
       menu.append(menusep)
@@ -556,7 +629,11 @@ class mixprefs:
       mp3_utf8_hbox = gtk.HBox()
       leftvbox.add(mp3_utf8_hbox)
       mp3_utf8_hbox.show()
+      speed_variance_box = gtk.HBox()
+      leftvbox.add(speed_variance_box)
+      speed_variance_box.show()
       leftvbox.show()
+      
       # mid column
       stream_vu_hbox = gtk.HBox()
       midvbox.add(stream_vu_hbox)
@@ -586,6 +663,10 @@ class mixprefs:
       dither_hbox.show()
       rightvbox.show()
 
+      self.speed_variance = gtk.CheckButton(speed_variance_text)
+      speed_variance_box.pack_start(self.speed_variance, False, False, 5)
+      self.speed_variance.connect("toggled", self.cb_pbspeed)
+      self.speed_variance.show()
       self.ask_profile = gtk.CheckButton(ask_profile_text)
       ask_profile_hbox.pack_start(self.ask_profile, False, False, 0)
       self.ask_profile.show()
@@ -634,7 +715,6 @@ class mixprefs:
       self.dither = gtk.CheckButton(dither_toggle_text)
       dither_hbox.pack_start(self.dither, False, False, 0)
       self.dither.connect("toggled", self.cb_dither)
-      self.dither.set_active(True)
       self.dither.show()
       self.keeppass = gtk.CheckButton(keeppass_text)
       keeppass_box.pack_start(self.keeppass, False, False, 0)
@@ -667,6 +747,46 @@ class mixprefs:
       outervbox.pack_start(frame, False, False, 0)
       frame.show()	# maybe this is just too geeky so I hide it
       
+      # User can use this to set the resampled sound quality
+      
+      frame = gtk.Frame(" " + player_resample_mode_text + " ")
+      frame.set_border_width(3)
+      hbox = gtk.HBox()
+      hbox.set_border_width(5)
+      frame.add(hbox)
+      hbox.show()
+      self.best_quality_resample = gtk.RadioButton(None, best_quality_resample_text)
+      self.best_quality_resample.connect("toggled", self.cb_resample_quality, 0)
+      rsbox = gtk.HBox()
+      rsbox.pack_start(self.best_quality_resample, True, False, 0)
+      rsbox.show()
+      hbox.add(rsbox)
+      self.best_quality_resample.show()
+      self.good_quality_resample = gtk.RadioButton(self.best_quality_resample, good_quality_resample_text)
+      self.good_quality_resample.connect("toggled", self.cb_resample_quality, 1) 
+      rsbox = gtk.HBox()
+      rsbox.pack_start(self.good_quality_resample, True, False, 0)
+      rsbox.show()
+      hbox.add(rsbox)
+      self.good_quality_resample.show()
+      self.fast_resample = gtk.RadioButton(self.good_quality_resample, fast_resample_text)
+      self.fast_resample.connect("toggled", self.cb_resample_quality, 2) 
+      rsbox = gtk.HBox()
+      rsbox.pack_start(self.fast_resample, True, False, 0)
+      rsbox.show()
+      hbox.add(rsbox)
+      self.fast_resample.show()
+      self.fastest_resample = gtk.RadioButton(self.fast_resample, fastest_resample_text)
+      self.fastest_resample.connect("toggled", self.cb_resample_quality, 4) 
+      rsbox = gtk.HBox()
+      rsbox.pack_start(self.fastest_resample, True, False, 0)
+      rsbox.show()
+      hbox.add(rsbox)
+      self.fastest_resample.show()
+      
+      outervbox.pack_start(frame, False, False, 0)
+      frame.show()
+      
       sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
       
       boostbox = gtk.HBox()
@@ -1228,6 +1348,7 @@ class mixprefs:
       
       # Mic mix and filter options
       
+      self.mic_filter = 0		# The type of filter to use for the mic filter
       frame = gtk.Frame(" " + af_h_sm_text + " ")
       frame.set_border_width(3)
       vbox = gtk.VBox()
@@ -1663,6 +1784,60 @@ class mixprefs:
       jack_vbox.show()
       jacklabel.show()
       
+      # event tab
+      
+      eventtabtips = gtk.Tooltips()
+      eventtabtips.enable()
+      
+      vbox = gtk.VBox()
+      vbox.set_border_width(4)
+      vbox.set_spacing(2)
+      
+      app_event_container = self.event_command_container()
+      self.appstart_event = self.event_command("icon", 20, 20, "amixer set Mic 0% mute cap ; " +
+		'amixer set Capture 50% cap ; amixer set "Mic Boost (+20dB)" on', False, False, eventtabtips, app_start_tip_text, shell_commands_tip_text)
+      app_event_container.add(self.appstart_event)
+      self.appstart_event.show()
+      self.appexit_event = self.event_command("icon", 20, 20, "", False, True, eventtabtips, app_exit_tip_text, shell_commands_tip_text)
+      app_event_container.add(self.appexit_event)
+      self.appexit_event.show()
+      vbox.pack_start(app_event_container, False, False, 0)
+      app_event_container.show()
+      
+      mic_event_container = self.event_command_container()
+      self.mic_on_event = self.event_command("mic4", 20, 20, "", False, False, eventtabtips, mic_on_tip_text, shell_commands_tip_text)
+      mic_event_container.add(self.mic_on_event)
+      self.mic_on_event.show()
+      self.mic_off_event = self.event_command("mic4", 20, 20, "", False, True, eventtabtips, mic_off_tip_text, shell_commands_tip_text)
+      mic_event_container.add(self.mic_off_event)
+      self.mic_off_event.show()
+      vbox.pack_start(mic_event_container, False, False, 0)
+      mic_event_container.show()
+      
+      aux_event_container = self.event_command_container()
+      self.aux_on_event = self.event_command("jack2", 20, 20, "", False, False, eventtabtips, aux_on_tip_text, shell_commands_tip_text)
+      aux_event_container.add(self.aux_on_event)
+      self.aux_on_event.show()
+      self.aux_off_event = self.event_command("jack2", 20, 20, "", False, True, eventtabtips, aux_off_tip_text, shell_commands_tip_text)
+      aux_event_container.add(self.aux_off_event)
+      self.aux_off_event.show()
+      vbox.pack_start(aux_event_container, False, False, 0)
+      aux_event_container.show()
+      
+      mutex_container = self.event_command_container()
+      self.mic_aux_mutex = gtk.CheckButton(mic_aux_mutex_text)
+      mutex_container.add(self.mic_aux_mutex)
+      self.mic_aux_mutex.show()
+      vbox.pack_start(mutex_container, False, False, 0)
+      mutex_container.show()
+      
+      eventlabel = gtk.Label(event_tab_text)
+      self.notebook.append_page(vbox, eventlabel)
+      eventlabel.show()
+      vbox.show()
+      
+      # about tab
+      
       frame = gtk.Frame()
       frame.set_border_width(9)
       vbox = gtk.VBox()
@@ -1709,6 +1884,8 @@ class mixprefs:
       self.enhanced_crossfade.set_active(True)
       self.djalarm.set_active(True)
       self.extm3u.set_active(True)
+      self.dither.set_active(True)
+      self.fastest_resample.set_active(True)
       
       self.load_jack_port_settings()
       self.bind_jack_ports()
@@ -1755,7 +1932,19 @@ class mixprefs:
          "dither"	 : self.dither,
          "recallsession" : self.restore_session_option,
          "MP3usesUTF8"   : self.mp3_utf8,
-         "keeppass"      : self.keeppass
+         "keeppass"      : self.keeppass,
+         "ee_appstart"	 : self.appstart_event,
+         "ee_appexit"    : self.appexit_event,
+         "ee_micon"	 : self.mic_on_event,
+         "ee_micoff"	 : self.mic_off_event,
+         "ee_auxon"	 : self.aux_on_event,
+         "ee_auxoff"	 : self.aux_off_event,
+         "best_rs"	 : self.best_quality_resample,
+         "good_rs"	 : self.good_quality_resample,
+         "fast_rs"	 : self.fast_resample,
+         "fastest_rs"	 : self.fastest_resample,
+         "micauxmutex"	 : self.mic_aux_mutex,
+         "speed_var"	 : self.speed_variance
 	 }
 	 
       self.valuesdict = {
@@ -1805,9 +1994,13 @@ class mixprefs:
 	 "timerchan"	 : self.timerchannelsentry,
 	 "timermess"	 : self.timermessageentry,
          "ltfilerqdir"   : self.parent.player_left.file_requester_start_dir,
-         "rtfilerqdir"   : self.parent.player_right.file_requester_start_dir
-	 }
-
-      self.mic_filter = 0		# The type of filter to use for the mic filter
+         "rtfilerqdir"   : self.parent.player_right.file_requester_start_dir,
+         "et_appstart"	 : self.appstart_event,
+         "et_appexit"    : self.appexit_event,
+         "et_micon"	 : self.mic_on_event,
+         "et_micoff"	 : self.mic_off_event,
+         "et_auxon"	 : self.aux_on_event,
+         "et_auxoff"	 : self.aux_off_event
+         }
 
       self.rangewidgets = (self.parent.deckadj,)
\ No newline at end of file
