@@ -19,7 +19,11 @@
 
 import pygtk
 pygtk.require('2.0')
-import gtk, os, eyeD3, sys
+import gtk, os, sys
+try:
+   import eyeD3
+except:
+   eyeD3 = None
 from IDJCfree import *
 from idjc_config import *
 from langheader import *
