@@ -321,11 +321,15 @@ class server_window:
          self.connect_button.set_active(True)
       if data == "connect":
          if widget.get_active() == True:
+            if self.format == "mp3" and os.environ["lame"] == "disabled":
+               print "lame is not installed - cannot stream in mp3 format"
+               self.nolame_howlame_dialog.present()
+               self.connect_button.set_active(False)
+               return
             self.dialog_group.hide()
             self.connect()
          else:
             self.disconnect()
-
             
    def cb_update(self, widget):
       print "Update was pressed"
@@ -476,6 +480,11 @@ class server_window:
       filename = unicode(self.recfile.get_text()).strip().encode(self.parent.fenc, "replace")
       format = self.rec_mode.get_active()
       if format <= 5:
+         if os.environ["lame"] == "disabled":
+            print "lame is not installed - cannot record in mp3 format"
+            self.nolame_howlame_dialog.present()
+            self.record_button.set_active(False)
+            return
          filetype = "mp3"
          bitrate = mp3bitratelist[format]
          if filename[-4:] != ".mp3" and filename[-4:] != ".MP3":
@@ -1051,6 +1060,7 @@ class server_window:
       self.autoshutdown_dialog = error_notification_dialog(self.dialog_group, self.parent.window_group, disconnected_text, autoshutdown_text)
       self.oneminwarn = autodisconnection_notification_dialog(self.dialog_group, self.parent.window_group, disconnection_text, discon_warn_text, None, self.cancel_autoshutdown)
       self.autoreconnect_dialog = error_notification_dialog(self.dialog_group, self.parent.window_group, reconnected_text, reconnected_additional_text)
+      self.nolame_howlame_dialog = error_notification_dialog(self.dialog_group, self.parent.window_group, missing_lame_text, to_install_lame_text)
 
       # Launch the server connection client
       (self.serv_cmd, self.serv_rply) = os.popen2([libexecdir + "idjcserver"], 16384)
