@@ -16,12 +16,12 @@
 #   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 appname= "Internet DJ Console"
-copyright = "Copyright 2005-2006 Stephen Fairchild"
-copyright2 = u"\xA9 2005-2006 Stephen Fairchild"	# As above but uses copyright character (c)
+copyright = "Copyright 2005-2007 Stephen Fairchild"
+copyright2 = u"\xA9 2005-2007 Stephen Fairchild"	# As above but uses copyright character (c)
 license = "Released under the GNU General Public License V2.0"
 # version is set in the configure.in file
 
-import locale, os, sys
+import locale, os, sys, fcntl
 
 def locale_encoding():
    read = os.popen("locale -a 2>/dev/null")# I have to do all this because an exception 
@@ -345,6 +345,7 @@ class select_profile_dialog(gtk.Dialog):
       if event.keyval == 65288: return False
       if event.keyval == 65293: return False
       if event.string.isalnum(): return False
+      if event.string == "_": return False
       return True
    
    def comboselection(self, widget):
@@ -493,7 +494,7 @@ class idjc_shutdown_dialog:
   
 class MainWindow:
    def send_new_mixer_stats(self):
-      string_to_send = ":%03d:%03d:%03d:%03d:%03d:%d:%d%d%d%d%d:%d%d:%d%d%d:%d%d%d%d:%d:%d:%d:%d:%d:" % (
+      string_to_send = ":%03d:%03d:%03d:%03d:%03d:%d:%d%d%d%d%d:%d%d:%d%d%d:%d%d%d%d:%d:%d:%d:%d:%d:%f:%f:%d:" % (
 						self.deckadj.get_value(),
 						self.crossadj.get_value(),
                                                 self.jingles.deckadj.get_value(),
@@ -519,7 +520,10 @@ class MainWindow:
                                                 self.mixermode,
                                                 self.prefs_window.fadeout.get_active(),
                                                 self.player_left.play.get_active() or
-                                                self.player_right.play.get_active())
+                                                self.player_right.play.get_active(),
+                                                1.0 / self.player_left.pbspeedfactor,
+                                                1.0 / self.player_right.pbspeedfactor,
+                                                self.prefs_window.speed_variance.get_active())
       self.mixer_write("MIXR=%s\nACTN=mixstats\nend\n" % string_to_send, True)
 
       self.alarm = False
@@ -610,7 +614,7 @@ class MainWindow:
    def callback(self, widget, data):
       print "%s was pressed" % data
       if data == "Show about":
-         self.prefs_window.notebook.set_current_page(4)
+         self.prefs_window.notebook.set_current_page(5)
          self.prefs_window.window.present()
       if data == "Features":
          if widget.get_active():
@@ -704,12 +708,42 @@ class MainWindow:
       return True
 	    
    def cb_toggle(self, widget, data):
-      print "Hello there, %s was toggled %s" % (data, ("OFF","ON")[widget.get_active()])
+      print "%s was toggled %s" % (data, ("OFF","ON")[widget.get_active()])
       if data == "Aux Open":
+         if widget.get_active() == False:
+            self.prefs_window.aux_off_event.activate()
+         else:
+            if self.prefs_window.mic_aux_mutex.get_active() and (self.mic_lselect.get_active or self.mic_rselect.get_active() or self.mixer_mode != self.NO_PHONE):
+               self.prefs_window.mic_aux_mutex.set_active(False)	# prevent infinite loop
+               self.greenphone.set_active(False)
+               self.redphone.set_active(False)
+               self.mic_select.set_active(False)
+               self.mic_lselect.set_active(False)
+               self.mic_rselect.set_active(False)
+               self.prefs_window.mic_aux_mutex.set_active(True)
+            self.prefs_window.aux_on_event.activate()
          self.send_new_mixer_stats()
       if data == "Mic L Open":
+         if self.mic_lselect.get_active() == False and self.mic_rselect.get_active() == False:
+            self.prefs_window.mic_off_event.activate()
+         else:
+            if self.prefs_window.mic_aux_mutex.get_active() and self.aux_select.get_active():
+               self.prefs_window.mic_aux_mutex.set_active(False)
+               self.aux_select.set_active(False)
+               self.prefs_window.mic_aux_mutex.set_active(True)
+            if self.mic_lselect.get_active() == True and self.mic_rselect.get_active() == False:
+               self.prefs_window.mic_on_event.activate()
          self.new_mixermode(self.mixermode)
       if data == "Mic R Open":
+         if self.mic_lselect.get_active() == False and self.mic_rselect.get_active() == False:
+            self.prefs_window.mic_off_event.activate()
+         else:
+            if self.prefs_window.mic_aux_mutex.get_active() and self.aux_select.get_active():
+               self.prefs_window.mic_aux_mutex.set_active(False)
+               self.aux_select.set_active(False)
+               self.prefs_window.mic_aux_mutex.set_active(True)
+            if self.mic_lselect.get_active() == False and self.mic_rselect.get_active() == True:
+               self.prefs_window.mic_on_event.activate()
          self.new_mixermode(self.mixermode)
       if data == "Mic Open":
          x = self.mic_select.get_active()
@@ -720,6 +754,10 @@ class MainWindow:
       if data == "Greenphone":
          mode = self.mixermode
          if widget.get_active() == True:
+            if self.prefs_window.mic_aux_mutex.get_active() and self.aux_select.get_active():
+               self.prefs_window.mic_aux_mutex.set_active(False)
+               self.aux_select.set_active(False)
+               self.prefs_window.mic_aux_mutex.set_active(True)
             if self.mixermode == self.PRIVATE_PHONE:
                self.mixermode = self.PUBLIC_PHONE
                self.redphone.set_active(False)
@@ -732,6 +770,10 @@ class MainWindow:
       if data == "Redphone":
          mode = self.mixermode
          if widget.get_active() == True:
+            if self.prefs_window.mic_aux_mutex.get_active() and self.aux_select.get_active():
+               self.prefs_window.mic_aux_mutex.set_active(False)
+               self.aux_select.set_active(False)
+               self.prefs_window.mic_aux_mutex.set_active(True)
             if self.mixermode == self.PUBLIC_PHONE:
                self.mixermode = self.PRIVATE_PHONE
                self.greenphone.set_active(False)
@@ -868,6 +910,7 @@ class MainWindow:
          self.prefs_window.save_player_prefs()
       try:
          gtk.main_quit()
+         os.unlink(self.idjcroot + "command")
       except:
          pass
       gtk.gdk.threads_leave()
@@ -910,9 +953,15 @@ class MainWindow:
       self.server_window.serv_cmd.close()
       self.mixer_ctrl.close()
       self.server_window.save(self.idjc)
- 
+      self.prefs_window.appexit_event.activate()	# run user specified command for application exit
+      if os.path.isfile(self.idjcroot + "command"):
+         os.unlink(self.idjcroot + "command")
       gtk.main_quit()
       time.sleep(0.35)	# Allow time for all subthreads/programs time to exit 
+      try:
+         os.unlink(self.idjcroot + "command")			# intentionally did this twice
+      except:
+         pass
       gtk.gdk.threads_leave()
       sys.exit(0)
 
@@ -1003,14 +1052,126 @@ class MainWindow:
          self.mixer_ctrl.close()
       return line
 
+   def process_play_command(self, filelist):
+      if self.crossadj.value < 50:
+         player = self.player_left
+      else:
+         player = self.player_right
+      player.stop.clicked()
+      player.liststore.clear()
+      self.process_enqueue_command(filelist)
+      player.play.clicked()
+   
+   def process_enqueue_command(self, filelist):
+      if self.crossadj.value < 50:
+         player = self.player_left
+      else:
+         player = self.player_right
+      for each in filelist:
+         if each.endswith(".m3u"):
+            filelist2 = self.process_m3u_file(each)
+            for item in filelist2:
+               self.append_item_to_playlist(player, item)
+         if each.endswith(".pls"):
+            filelist2 = self.process_pls_file(each)
+            for item in filelist2:
+               self.append_item_to_playlist(player, item)
+         else:
+            self.append_item_to_playlist(player, each)
+            
+   def process_m3u_file(self, filename):
+      contents = []
+      try:
+         file = open(filename, "r")
+      except IOError:
+         print "unable to open file", filename
+      else:
+         while 1:
+            line = file.readline()
+            if not line:
+               break
+            if line[0] != "#":
+               contents.append(line[:-1])
+      return contents
+   
+   def process_pls_file(self, filename):
+      contents = []
+      try:
+         file = open(filename, "r")
+      except IOError:
+         print "unable to open file", filename
+      else:
+         while 1:
+            line = file.readline()
+            if not line:
+               break
+            if line.startswith("File"):
+               line = line.split("=")
+               contents.append(line[1][:-1])
+      return contents
+   
+   def append_item_to_playlist(self, player, file):
+      entryline = player.get_media_metadata(file)
+      if entryline[0] != "Not a valid file":
+         player.liststore.append(entryline)
+      else:
+         print "file not valid:", file
+
+   def process_command_file(self):
+      try:
+         file = open(self.idjcroot + "command", "r+")
+      except IOError:
+         print "Problem reading command file from disk"
+      else:
+         fcntl.flock(file.fileno(), fcntl.LOCK_EX)
+         class stop(Exception): pass
+         try:
+            while 1:
+               line = file.readline()
+               if not line:		# check if we are finished
+                  break
+               if line[0] != "[":	# 1st line must be of the form
+                  raise stop		# [--command x] where x is the number of lines that follow
+               if line[-2:] != "]\n":
+                  raise stop
+               line = line[1:-2].split(" ")
+               if not line[0].startswith("--"):
+                  raise stop
+               else:
+                  command = line[0][2:]
+               try:
+                  numberoffiles = int(line[1])
+               except:
+                  raise stop
+               listoffiles = []
+               for item in range(numberoffiles):
+                  filename = file.readline()
+                  if filename == "":	# check the correct number of files are present
+                     raise stop
+                  listoffiles.append(filename[:-1])
+               if command == "enqueue":
+                  self.process_enqueue_command(listoffiles)
+               elif command == "play":
+                  self.process_play_command(listoffiles)
+               else:
+                  print "unknown command", command
+         except stop:
+            print "corruption detected in command file\n"
+         file.truncate(0)
+         fcntl.flock(file.fileno(), fcntl.LOCK_UN)
+         file.close()
+
    def vu_update(self, locking = True):
       if locking:
          gtk.gdk.threads_enter()
          
       if self.second_instance:
-         self.second_instance = False
          self.window.present()
-         
+         self.second_instance = False
+      
+      if os.stat(self.idjcroot + "command").st_size:
+         self.process_command_file()
+      
       try:
          self.mixer_write("ACTN=requestlevels\nend\n", True)
       except ValueError, IOError:
@@ -1088,12 +1249,32 @@ class MainWindow:
          widget.popup(None, None, None, event.button, event.time)
          return True
       return False
-
+   
+   def close_window(self, widget, menu):
+      menu.get_attach_widget().hide()
+   
+   def close_menu_attach(self, widget, event, data):
+      def detach(self, widget):
+         pass
+      if event.button == 3:
+         if self.close_menu.get_attach_widget() is not None:
+            self.close_menu.detach()
+         self.close_menu.attach_to_widget(data, detach)
+         self.close_menu.popup(None, None, None, event.button, event.time)
+   
    # when a second instance of idjc is launched the program launcher will signal the running instance with SIGUSR2
    def second_instance_handler(self, arg1, arg2):
       print "the launch of a second instance of idjc was detected"
       self.second_instance = True	# idjc will handle this event in good time
 
+   def profiletest(self, profile):
+      for each in profile:
+         if each.isalnum() or each == "_":
+            continue
+         else:
+            return False
+      return True
+
    class initfailed:
       def __init__(self, errormessage = "something bad happened and IDJC could not continue"):
          print errormessage
@@ -1102,15 +1283,30 @@ class MainWindow:
       pass
 
    def __init__(self, clargs):
-      self.second_instance = False
-      signal.signal(signal.SIGUSR2, self.second_instance_handler)
-      signal.signal(signal.SIGINT, self.destroy_hard)
-      
       self.appname = appname
       self.version = version
       self.copyright = copyright2
       self.license = license
       
+      self.second_instance = False
+      signal.signal(signal.SIGUSR2, self.second_instance_handler)
+      signal.signal(signal.SIGINT, self.destroy_hard)
+      
+      home = os.environ.get("HOME")
+      if home == "":
+         raise self.initfailed("Error, HOME environment variable is not set")
+      # Build the application data directory tree for idjc
+      self.home = home
+      self.idjcroot = home + "/.idjc/"
+      if not os.path.isdir(home + "/.idjc"):
+         os.mkdir(home + "/.idjc")
+      if not os.path.isdir(home + "/.idjc/jingles"):
+         os.mkdir(home + "/.idjc/jingles")
+      if not os.path.isdir(home + "/.idjc/profiles"):
+         os.mkdir(home + "/.idjc/profiles")
+      if not os.path.exists(home + "/.idjc/profiles/default"):
+         os.symlink(home + "/.idjc/", home + "/.idjc/profiles/default")
+      
       print appname, "Version", version, "\n" + copyright, "\n" + license + "\n"
       arghelp = argver = False
       self.profile = ""
@@ -1121,11 +1317,11 @@ class MainWindow:
             arghelp = True
       if argver == False and arghelp == False:
          if len(clargs) == 1:
-            if clargs[0].isalnum():
+            if self.profiletest(clargs[0]):
                self.profile = clargs[0]
                print "using profile:", self.profile
             else:
-               print "error: profile must be alphanumeric i.e. a-z A-Z 0-9"
+               print "error: profile must be alphanumeric i.e. a-z A-Z 0-9 underscores are allowed"
                raise self.initcleanexit
          else:
             if len(clargs) > 1:
@@ -1144,33 +1340,6 @@ class MainWindow:
       self.denc = display_enc
       self.fenc = file_enc
       print "display encoding:", self.denc, "filesystem encoding:", self.fenc
-      home = os.environ.get("HOME")
-      if home == "":
-         raise self.initfailed("Error, HOME environment variable is not set")
-      
-      # Build the application data directory tree for idjc
-      self.home = home
-      self.idjcroot = home + "/.idjc/"
-      if not os.path.isdir(home + "/.idjc"):
-         os.mkdir(home + "/.idjc")
-      if not os.path.isdir(home + "/.idjc/jingles"):
-         os.mkdir(home + "/.idjc/jingles")
-      if not os.path.isdir(home + "/.idjc/profiles"):
-         os.mkdir(home + "/.idjc/profiles")
-      if not os.path.exists(home + "/.idjc/profiles/default"):
-         os.symlink(home + "/.idjc/", home + "/.idjc/profiles/default")
-      # Generate xine config file as needed
-      if not os.path.exists(home + "/.idjc/xine-config"):
-	 xconfig = None
-	 try:
-	    try:
-	       xconfig = open(home + "/.idjc/xine-config", "w")
-	       xconfig.write("# xine config file generated by idjc\n\n.version:2\n\naudio.driver:none\nvideo.driver:none\nengine.performance.memcpy_method:mmx\n")
-	    except:
-	       print "failed generation of xine-config file"
-	 finally:
-	    if xconfig:
-	       xconfig.close()
       
       ask_profile = not os.path.isfile(self.idjcroot + "do-not-ask-profile")
       if self.profile == "":
@@ -1227,10 +1396,9 @@ class MainWindow:
                pid.destroy()
       else:
          self.idjc = home + "/.idjc/profiles/" + self.profile + "/"
-
+      
       while gtk.events_pending():	# update the GUI rendering to reflect the closure of the window
-         gtk.main_iteration()
-	 
+         gtk.main_iteration() 
 
       self.session_loaded = False
       self.mixer_ctrl, self.mixer_rply = os.popen2([ libexecdir + "idjcmixer" ], 4096)
@@ -1245,7 +1413,7 @@ class MainWindow:
       while rply[:6] != "IDJC: ":
          if rply == "":
             print "mixer crashed"
-            message_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "The mixer module crashed during initialisation.\nThe most likely cause is a corrupt xine config file.\nDeleting the xine config files should solve the problem if relaunching IDJC should fail repeatedly.\ne.g. rm -fdr ~/.xine/")
+            message_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "The mixer module crashed during initialisation.")
             message_dialog.set_icon_from_file(pkgdatadir + "icon" + gfext)
             message_dialog.run()
             message_dialog.destroy()
@@ -1331,10 +1499,10 @@ class MainWindow:
       
       # do box 7 before the others since it is nice and simple
       # add a playlist button to open the playlist window
-      self.button = gtk.Button(prefs_button_text)
-      self.button.connect("clicked", self.callback, "Prefs")
-      self.hbox7.pack_start(self.button, True, True, 0)
-      self.button.show()
+      self.prefs_button = gtk.Button(prefs_button_text)
+      self.prefs_button.connect("clicked", self.callback, "Prefs")
+      self.hbox7.pack_start(self.prefs_button, True, True, 0)
+      self.prefs_button.show()
       
       # add a server button to open the server window
       # this incorporates some led type indicators
@@ -1845,6 +2013,15 @@ class MainWindow:
       menu_jingles.show()
       view_menu.append(menu_jingles)
 
+      # menu for closing prefs, server, jingles
+
+      self.close_menu = gtk.Menu()
+      close_menu_item = gtk.MenuItem(window_close_text)
+      self.close_menu.add(close_menu_item)
+      self.close_menu.show()
+      close_menu_item.show()
+      close_menu_item.connect("activate", self.close_window, self.close_menu)
+
       # Instantiate the other windows
       self.songname = u""
       self.newmetadata = False
@@ -1884,6 +2061,18 @@ class MainWindow:
       self.in_vu_timeout = False
       self.vucounter = 0
       self.session_filename = self.idjc + "main_session"
+      
+      # create or empty out the command file which should not exist when idjc is not running
+      try:
+         file = open(self.idjcroot + "command", "w")
+      except IOError:
+         print "unable to initialise the command file"
+         raise self.initfailed
+      else:
+         fcntl.flock(file.fileno(), fcntl.LOCK_EX)
+         file.truncate(0)
+         fcntl.flock(file.fileno(), fcntl.LOCK_UN)
+         file.close()
 
       # Variable map for stuff read from the mixer
       self.vumap = {
@@ -1935,6 +2124,12 @@ class MainWindow:
          self.restore_session()
       self.session_loaded = True
        
+      self.prefs_button.connect("button-press-event", self.close_menu_attach, self.prefs_window.window)
+      self.server_button.connect("button-press-event", self.close_menu_attach, self.server_window.window)
+      self.jingles_button.connect("button-press-event", self.close_menu_attach, self.jingles.window)
+       
+      self.prefs_window.appstart_event.activate()	# run user specified commands for application start
+       
       self.window.show()
    
    def main(self):
@@ -1942,19 +2137,19 @@ class MainWindow:
 
 def environment_variables_okay():
    # Shows a dialog box when one or more of the helper programs is missing
-   df = "/bin/true"		# default program to use in place of the missing
    environvars = (
       ("metaflac from the flac package needed to play flac files\n", ("metaflac", "missing")),
-      ("lame needed for the encoding of mp3 streams\n", ("lame", df)),
-      ("oggenc needed for the encoding of ogg/vorbis\n", ("oggenc", df)),
+      ("lame needed for the encoding of mp3 streams\n", ("lame", "disabled")),
+      ("oggenc needed for the encoding of ogg/vorbis\n", ("oggenc", "missing")),
       ("ogginfo needed for ogg track length and tags\n", ("ogginfo", "missing")),
       ("jackd from the jack-audio-connection-kit package\n", ("jackd", "missing")),
       ("vorbiscomment from the vorbistools package\n", ("vorbiscomment", "missing")) )
    error_message = "The following programs were found to be missing...\n\n"
    for each in environvars:
-      if os.environ.get(each[1][0], each[1][1]) == each[1][1]:
-         error_message = error_message + each[0]	# append the appropriate error message
-         os.environ[each[1][0]] = each[1][1]		# set the actual environment variable to the chosen default
+      if os.environ.get(each[1][0], "") == "":
+         if each[1][1] == "missing":
+            error_message = error_message + each[0]	# append the appropriate error message
+         os.environ[each[1][0]] = each[1][1]
    if error_message[-2:] != "\n\n":
       error_message = error_message + "\n\t\tDo you want to continue?"
       message_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, error_message)
