@@ -51,6 +51,7 @@ server_button_text = ("Server", "Server", "Serveur")[li]
 server_window_text = ("Radio Server", "Radioserver", "Radio Server")[li]
 jingles_button_text = ("Jingles", "Jingles", "Jingles")[li]
 jingles_window_text = ("IDJC Jingles", "IDJC Jingles", "IDJC Jingles")[li]
+window_close_text = ("Close", "Close", "Close")[li]
 
 # Jingles window stuff
 
@@ -85,6 +86,7 @@ record_title_text = ("Title:", "Titel:", "Title:")[li]
 general_tab_text = ("General", "Grundeinstellungen", "General")[li]
 microphone_tab_text = ("Microphone", "Mikrophon", "Microphone")[li]
 jack_ports_tab_text = ("Jack Ports", "Jack-Anschlüsse", "Jack Ports")[li]
+event_tab_text = ("Event", "Event", "Event")[li]
 about_tab_text = ("About", "Über", "About")[li]
 
 # Stuff in the general tab of prefs window
@@ -111,6 +113,7 @@ fadeout_toggle_text = ("Fadeout", "Fadeout", "Fadeout")[li]
 dj_alarm_toggle_text = ("DJ Alarm", "DJ Alarm", "DJ Alarm")[li]
 dither_toggle_text = ("Dither", "Dither", "Dither")[li]
 keeppass_text = ("Keep Password", "Keep Password", "Keep Password")[li]
+speed_variance_text = ("PB Speed Adj.", "PB Speed Adj.", "PB Speed Adj.")[li]
 initial_player_settings_text = ("Player Settings At Startup", "Einstellungen nach Programmstart", "Player Settings At Startup")[li]
 player_1_text = ("Player 1", "Player 1", "Player 1")[li]
 player_2_text = ("Player 2", "Player 2", "Player 2")[li]
@@ -178,6 +181,13 @@ jack_help_text = (
 "Type <span foreground='red'>jack_lsp</span> in a console for a list of available ports.",
 "Führe <span foreground='red'>jack_lsp</span> in einer Konsole aus, um eine Liste der verfügbaren Anschlüsse zu erhalten.",
 "Type <span foreground='red'>jack_lsp</span> in a console for a list of available ports.")[li]
+
+# Event tab
+
+mic_aux_mutex_text = ("Make Mic and Aux buttons mutually exclusive", "Make Mic and Aux buttons mutually exclusive", "Make Mic and Aux buttons mutually exclusive")[li]
+
+# About tab
+
 licence_text = ("Released under the GNU General Public License V2.0",
 		"Veröffentlicht unter der GNU General Public License V2.0",
 		"Released under the GNU General Public License V2.0")[li]
@@ -242,6 +252,9 @@ disconnection_text = ("IDJC Disconnection", "IDJC trennt die Verbindung", "IDJC
 
 discon_warn_text = (("You will be automatically disconnected from", "the server one minute from now."), ("Sie werden in einer Minute", "automatisch vom Server getrennt."), ("You will be automatically disconnected from", "the server one minute from now."))[li]
 
+missing_lame_text = ("MP3 Not available", "MP3 Not available", "MP3 Not available")[li]
+to_install_lame_text = (("LAME needs to be installed", "in order to stream or record", "in mp3 format."), ("LAME needs to be installed", "in order to stream or record", "in mp3 format."), ("LAME needs to be installed", "in order to stream or record", "in mp3 format."))[li]
+
 autopause_text = ("Record Autosuspend", "Automatische Aufnahmepause", "Record Autosuspend")[li]
 ap_on_delay_text = ("Delay", "Verzögerung", "Delay")[li]
 ap_off_thresh_text = ("Thresh.", "Threshold", "Threshold")[li]
@@ -256,6 +269,8 @@ right_playlist_save_text = ("Save right playlist", "Rechte Abspielliste speicher
 
 # Playlist control elements
 
+normal_speed_element_text = (">> Normal Speed <<", ">> Normal Speed <<", ">> Normal Speed <<")[li]
+
 stop_control_element_text = ("Player stop", "Player anhalten", "Player stop")[li]
 
 transfer_control_ltr_element_text = (">>> Transfer across >>>", ">>> Zum rechten Player springen >>>", ">>> Transfer across >>>")[li]
@@ -270,8 +285,12 @@ stream_disconnect_element_text = ("Stop streaming", "Streaming anhalten", "Stop
 
 stop_recording_element_text = ("Stop recording", "Aufnahme anhalten", "Stop recording")[li]
 
+open_aux_element_text = ("Switch to Aux input", "Switch to Aux input", "Switch to Aux input")[li]
+
 # The menu items for the playlist control elements
 
+normal_speed_control_menu_text = ("Normal Speed", "Normal Speed", "Normal Speed")[li]
+
 stop_control_menu_text = ("Player stop", "Player anhalten", "Player stop")[li]
 
 transfer_control_menu_text = ("Transfer", "Sprung zum anderem Player", "Transfer")[li]
@@ -284,6 +303,8 @@ stop_recording_menu_text = ("Stop recording", "Aufnahme anhalten", "Stop recordi
 
 control_menu_text = ("Insert control", "Steuerelement einfügen", "Insert control")[li]
 
+auxinput_control_menu_text = ("Switch to Aux", "Switch to Aux", "Switch to Aux")[li]
+
 # Stream encoding method
 
 format_text = ("Format", "Format", "Format")[li]
@@ -329,4 +350,29 @@ select_profile_body_text = ("Which profile do you wish to use?", "Which profile
 select_profile_new_text = ("A New Profile", "A New Profile", "A New Profile")[li]
 no_more_ask_text = ("Don't ask again", "Don't ask again", "Don't ask again")[li]
 
+# Tooltips
+
+shell_commands_tip_text = ("Bash shell commands to run", "Bash shell commands to run", "Bash shell commands to run")[li]
+app_start_tip_text = ("Application Start", "Application Start", "Application Start")[li]
+app_exit_tip_text = ("Application Close", "Application Close", "Application Close")[li]
+mic_on_tip_text = ("Microphone On", "Microphone On", "Microphone On")[li]
+mic_off_tip_text = ("Microphone Off", "Microphone Off", "Microphone Off")[li]
+aux_on_tip_text = ("Aux On", "Aux On", "Aux On")[li]
+aux_off_tip_text = ("Aux Off", "Aux Off", "Aux Off")[li]
+
+# Playlist window information box
+
+popupwindowplaying_text = ("Playing track %d of %d", "Playing track %d of %d", "Playing track %d of %d")[li]
+popupwindowtracktotal_text = ("Total number of tracks %d", "Total number of tracks %d", "Total number of tracks %d")[li]
+popupwindowplayduration_text = ("Total play duration %s", "Total play duration %s", "Total play duration %s")[li]
+
+# Playlist resample box
+
+player_resample_mode_text = ("Player resample quality", "Player resample quality", "Player resample quality")[li]
+best_quality_resample_text = ("Highest", "Highest", "Highest")[li]
+good_quality_resample_text = ("Good", "Good", "Good")[li]
+fast_resample_text = ("Fast", "Fast", "Fast")[li]
+fastest_resample_text = ("Fastest", "Fastest", "Fastest")[li]
+
+
 del li
